c laguer.f
c Numerical Recipes (http://www.nr.com), Section 9.5

      SUBROUTINE laguer(a,m,x,its)

      INTEGER m,its,MAXIT,MR,MT
      REAL EPSS
      COMPLEX a(m+1),x
      PARAMETER (EPSS=2.e-7,MR=8,MT=10,MAXIT=MT*MR)
c Given the degree m and the complex coefficients a(1:m+1) of the polynomial Sum_i=1^m+1 a(i) x^(i-1),
c and given a complex value x, this routine improves x by Laguerres method until it converges,
c within the achievable roundoff limit, to a root of the given polynomial. The number
c of iterations taken is returned as its.
c Parameters: EPSS is the estimated fractional roundoff error. We try to break (rare) limit
c cycles with MR different fractional values, once every MT steps, for MAXIT total allowed
c iterations.
      INTEGER iter,j
      REAL abx,abp,abm,err,frac(MR)
      COMPLEX dx,x1,b,d,f,g,h,sq,gp,gm,g2
      SAVE frac
c Fractions used to break a limit cycle.
      DATA frac /.5,.25,.75,.13,.38,.62,.88,1./
c Loop over iterations up to allowed maximum.
      do 12 iter=1,MAXIT
        its=iter
        b=a(m+1)
        err=abs(b)
        d=cmplx(0.,0.)
        f=cmplx(0.,0.)
        abx=abs(x)
        do 11 j=m,1,-1
c Efficient computation of the polynomial and its first
c two derivatives. f stores P''2.
          f=x*f+d
          d=x*d+b
          b=x*b+a(j)
          err=abs(b)+abx*err
11      enddo
c Estimate of roundoff error in evaluating polynomial.
        err=EPSS*err
        if (abs(b).le.err) then
c We are on the root.
          return
        else
c The generic case: use Laguerres formula.
          g=d/b
          g2=g*g
          h=g2-2.*f/b
          sq=sqrt((m-1)*(m*h-g2))
          gp=g+sq
          gm=g-sq
          abp=abs(gp)
          abm=abs(gm)
          if (abp.lt.abm) gp=gm
          if (max(abp,abm).gt.0.) then
            dx=m/gp
          else
            dx=exp(cmplx(log(1.+abx),float(iter)))
          endif
        endif
        x1=x-dx
        if (x.eq.x1) return
c Converged.
        if (mod(iter,MT).ne.0) then
          x=x1
        else
c Every so often we take a fractional step, to break any
c limit cycle (itself a rare occurrence).
          x=x-dx*frac(iter/MT)
        endif
12    enddo
      write(*,*) 'laguer.f: too many iterations in laguer'
c Very unusual - can occur only for complex roots.
c Try a different starting guess for the root.
      return
      END


